#include "rwconfig.h"
#include "skin.h"
#include "reciteword.h"
#include <cstdio>
#include <sys/stat.h>
#include <cstring>

#define DEFAULT_SKIN_DIR "bdc"

extern CReciteWord *g_pReciteWord;

ConfigFile *usercfgfile = NULL;

static void
conf_init ()
{
	struct stat st;
	gchar rw_dir[256];

#ifdef G_OS_WIN32
	FILE *file;
	if (stat ("users", &st))
		mkdir ("users");
	if (stat ("users\\default", &st))
		mkdir ("users\\default");
	strcpy (rw_dir, "users" G_DIR_SEPARATOR_S "reciteword.cfg");
	if (stat (rw_dir, &st))
	{
		if ((file = fopen (rw_dir, "w")) == NULL)
			return;
		fprintf (file, "[reciteword]\nuser=default\n");
		
		fclose (file);
	}
#else
	sprintf(rw_dir,"%s/.reciteword",g_get_home_dir());
	if (stat (rw_dir, &st))
		mkdir (rw_dir,00755);
#endif
}

static void
user_init (gchar * user_file)
{
	struct stat st;
	FILE *file;
	if (stat (user_file, &st))
	{
		if ((file = fopen (user_file, "w")) == NULL)
			return;
		fprintf (file,"[reciteword]\nskin=" DEFAULT_SKIN_DIR "\nnormal_english_font=Utopia Bold 10\nbig_english_font=Utopia Bold 20\n");
		fprintf (file,"[chooseword]\ndisorder=1\n");
		fprintf (file,"[dict]\nrtsearch=1\nreadword=1\n");
		fprintf (file,"[firstrecite_group]\ndisorder=1\n");
		fprintf (file,"[firstrecite_skim]\nreadword=1\nshowphonetic=1\nshowmeanlater=1\n");
		fprintf (file,"[firstrecite_test]\nshowmean_1=1\nreadword_1=1\nsndeffect_1=1\nshowmean_2=1\nsndeffect_2=1\n");
		fprintf (file,"[revise_group]\ndisorder=1\nmax_revise_times=6\n");
		fprintf (file,"[revise_test]\nshowmean_1=1\nreadword_1=1\nshowphonetic_1=1\nsndeffect_1=1\nshowmean_2=1\nsndeffect_2=1\n");
		fprintf (file,"[revise_skim]\nreadword=1\nshowmean=1\nanimate=1\n");
		fprintf (file,"[shooting]\nreadword=1\nshowword=1\nshotsnd=1\n");
		fprintf (file,"[typing]\nreadword=1\nshowmean=1\nsndeffect=1\nshowfingering=1\n");
		fclose (file);
	}
}

int
conf_load_user (gboolean onlyshowdict)
{
	gchar *ab, str[256], str2[256];

	conf_init ();
#ifdef G_OS_WIN32
	ConfigFile *cfgfile;
	cfgfile = rw_cfg_open_file ("users"G_DIR_SEPARATOR_S"reciteword.cfg");
	if (!cfgfile)
	{
		g_print ("load users"G_DIR_SEPARATOR_S"reciteword.cfg file error\n");
		return TRUE;
	}
	if (rw_cfg_read_string (cfgfile, "reciteword", "user", &ab))
	{
		sprintf (str,"users"G_DIR_SEPARATOR_S"%s", ab);
		g_free (ab);
		if (str[0] == '\0')
			strcpy (str, "users"G_DIR_SEPARATOR_S"default");
	}
	else
	{
		strcpy (str, "users"G_DIR_SEPARATOR_S"default");
	}
	rw_cfg_free (cfgfile);
	g_free (cfgfile);
#else
	sprintf(str,"%s/.reciteword",g_get_home_dir());
#endif

	strcpy(g_pReciteWord->userpath,str); //set the reciteword user's path.
	
	sprintf (str2, "%s"G_DIR_SEPARATOR_S"my.cfg", str);
	user_init (str2);
	
	usercfgfile = rw_cfg_open_file (str2);
	if (!usercfgfile)
	{
		g_print ("load file \"%s\" error!\n", str2);
		return TRUE;
	}

	if (rw_cfg_read_string (usercfgfile, "reciteword", "skin", &ab))
	{
		strcpy (str, ab);
		g_free (ab);
		if (str[0] == '\0')
			strcpy (str, DEFAULT_SKIN_DIR);
	}
	else
	{
		strcpy (str, DEFAULT_SKIN_DIR);
	}
	if (skin_load_info (str)) //load skin info fail
	{
		if (strcmp (str, DEFAULT_SKIN_DIR) == 0)
		{
			rw_cfg_free (usercfgfile);
			g_free (usercfgfile);
			usercfgfile = NULL;
			return TRUE;
		}
		else // if it is not the default skin,try default skin when failed.
		{
			if (skin_load_info (DEFAULT_SKIN_DIR))
			{
				rw_cfg_free (usercfgfile);
				g_free (usercfgfile);
				usercfgfile = NULL;
				return TRUE;
			}
			else //load default skin succeed.
			{
				gtk_rc_parse ("skins"G_DIR_SEPARATOR_S DEFAULT_SKIN_DIR G_DIR_SEPARATOR_S"gtkrc");
			}
		}
	}
	else  // load skin succeed.
	{
		sprintf (str2, "skins"G_DIR_SEPARATOR_S"%s"G_DIR_SEPARATOR_S"gtkrc", str);
		gtk_rc_parse (str2);
	}
	
	rw_cfg_read_boolean (usercfgfile, "reciteword", "disable_sound", &(g_pReciteWord->sndserv.disable));
	rw_cfg_read_boolean (usercfgfile, "reciteword", "disable_soundevent", &(g_pReciteWord->soundevent.disable));
	rw_cfg_read_boolean (usercfgfile, "reciteword", "disable_readword", &(g_pReciteWord->wyabdc.disable));
	
	gboolean use_custom_font;
	rw_cfg_read_boolean (usercfgfile, "reciteword", "use_custom_font", &use_custom_font);
	if (use_custom_font)
	{
		gchar *bb;
		if (rw_cfg_read_string (usercfgfile, "reciteword", "local_language_font", &bb))
		{
			gchar *aa;
			aa = g_strdup_printf("style \"custom-font\" { font_name= \"%s\" }\nclass \"GtkWidget\" style \"custom-font\"\n",bb);
			gtk_rc_parse_string(aa);
			g_free(bb);
			g_free(aa);
		}
	}
	
	if (onlyshowdict)  //needn't load book when only show dict window.
		return FALSE;
	if (rw_cfg_read_string (usercfgfile, "reciteword", "book_file", &ab))
	{
		g_pReciteWord->load_book(ab);
		g_free (ab);
	}
	
	return FALSE;
}

void conf_save_usr ()
{
	if ((usercfgfile)&&(usercfgfile->changed))
	{
		if (rw_cfg_write_file(usercfgfile,usercfgfile->cfgfilename))
		{
			usercfgfile->changed = false;
		}
		else
		{
			g_print("Write file \"%s\" failed!\n",usercfgfile->cfgfilename);
		}
	}
}
